<?php
//require_once '/../TeknisaAbstractRegra.php';

/**
 * CalculoRegra.
 *
 * @package Logic
 * @author  alessandrobartels / brunofeliciano - 12/08/2011
 */
class CalculoRegra extends TeknisaRegra{


    /** @var  $_itens_calculo FpaitecalcfolhaEntity*/
    protected $_itens_calculo;      //BORBA 01/07/2013

    private $_calculo;

    private $_mesOcorrenciaEventoCorrente;
    private $_referenciaEventoCorrente;
    private $_valorEventoCorrente;
    private $_ocorrenciasEventoCorrente;
    private $_nrEventoCorrente;

    private $_eventoCorrente;


    /**
     * Construtor.
     *
     * @param Calculo $calculo Instancia de Calculo.
     *
     * @author alessandrobartels / brunofeliciano - 12/08/2011
     */
    public function __construct(Calculo $calculo){

        $this->_calculo = $calculo;
        $folhaEvaluator = new FolhaEvaluator($this);
        parent::__construct($folhaEvaluator);
        /* $this->geraVariaveis(); */
    }


    /**
     * Gera as variaveis de regra.
     *
     * @return void
     *
     * @author alessandrobartels 12/08/2011
     */
    public function geraVariaveis(){
        $this->setVar('MES_COMPETENCIA', $this->_calculo->getMesCompetencia());
        $this->setVar('DIAS_TRABALHO', $this->_calculo->getDiasNormais());
        $this->setVar('DIAS_FERIAS', $this->_calculo->getDiasFerias());
        $this->setVar('DIAS_AFASTADO', $this->_calculo->getDiasAfastados());
        $this->setVar('JORNADA_DIARIA', $this->_calculo->getJornadaDiaria());
        // @todo Chamar os metodos do alteracao salario e nao do vinculo.
        /*  $this->setVar(
            'SALARIO_MENSAL', $this->_calculo->getVinculo()->historico->converteValorSalarioParaMensal(
                GpeparamoperEntity::IDSALARIOOPER_REAL, $this->_calculo->getJornadaDiaria()
            )
        );
        $this->setVar(
            'SALARIO_HORA', $this->_calculo->getVinculo()->historico->converteValorSalarioParaHora(
                GpeparamoperEntity::IDSALARIOOPER_REAL, $this->_calculo->getJornadaDiaria()
            )
        );
        */
        /* @todo Criar como funcao */
        /* $this->setVar('TOTALIZA_PROVENTO', $this->totalizaProvento());
        $this->setVar('TOTALIZA_DESCONTO', $this->totalizaDesconto()); */

    }

    public function retornaValorPorCondicao($condicao, $valorSeVerdadeiro, $valorSeFalso) {
        if ($condicao) {
            return $valorSeVerdadeiro;
        } else if ($valorSeFalso !== false && $valorSeFalso !== null) {
            return $valorSeFalso;
        } else {
            return ABORT;
        }
    }

    /**
     * Enter description here ...
     *
     * @param integer $nrEvento Numero do evento.
     *
     * @return void
     *
     * @author alessandrobartels 12/08/2011
     */
    private function _atribuiVariavelEventoCorrente($nrEvento){
        $this->setVar('EVENTO_CORRENTE', $nrEvento);
    }


    /**
     * Retorna o atributo Calculo.
     *
     * @return Calculo
     *
     * @author alessandrobartels 12/08/2011
     */
    public function getCalculo(){
        return $this->_calculo;
    }


    /* ##############################   INCIDENCIAS   ##############################*/


    /**
     * Retorna o valor totalizado dos eventos de uma incidencia no mes de competencia.
     *
     * @param integer $tipo_incidencia Numero do tipo de incidencia.
     *
     * @return integer
     *
     * @author alessandrobartels / bruno feliciano - 17/08/2011
     */
    public function retornaValorIncidencia($tipo_incidencia){
        $mes_competencia = $this->_calculo->getMesCompetencia();
        $valor = $this->_itens_calculo->calculaIncidencias(
            $tipo_incidencia, $mes_competencia, $this->_calculo->getEventom()
        );
        return $valor;
    }


    /**
     * Retorna o valor totalizados das referencias de uma incidencia no mes de competencia.
     *
     * @param integer $tipo_incidencia Numero do tipo de incidencia.
     *
     * @return integer
     *
     * @author alessandrobartels / bruno feliciano - 17/08/2011
     *         alessandrobartels - 28/06/2012 - Consolidado das referencias
     */
    public function retornaReferenciaIncidencia($tipo_incidencia){
        $mes_competencia = $this->_calculo->getMesCompetencia();
        /*$referencia = $this->_itens_calculo->calculaReferenciaIncidencias(
            $tipo_incidencia, $mes_competencia, $this->_calculo->getEventom()
        );*/
        /* 28/06/12 - O consolidade das referencias eh gerado dentro do calculo de incidencias.
         *            Nao eh mais um metodo separado.
         */
        $valor = $this->_itens_calculo->calculaIncidencias(
            $tipo_incidencia, $mes_competencia, $this->_calculo->getEventom()
        );
        $referencia = $this->_itens_calculo->getTotalReferencias();
        return $referencia;
    }


    /* ##############################   ITENS CALCULOS (EVENTOS)   ##############################*/


    /**
     * Retorna o valor do evento no mes de competencia.
     *
     * @param integer $nr_evento Numero do evento.
     *
     * @return float
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     */
    public function retornaValorEvento($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $valor = $this->_itens_calculo->retornaValorPorEvento($evento);
        return $valor;
    }


     /**
     * Retorna a referencia do evento no mes de competencia.
     *
     * @param integer $nr_evento Numero do evento.
     *
     * @return integer
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     */
    public function retornaReferenciaEvento($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $referencia = $this->_itens_calculo->retornaReferenciaPorEvento($evento);
        return $referencia;
    }


    /**
     * Retorna numero de dias da competencia atual 28, 30 ou 31
     *
     * @return integer
     *
     * @author brunoborba - 21/07/2014
     */
    public function getDiasCompetencia(){
        $dia_inicial=$this->getDataCompetencia();
        $dia_final=$this->getUltimoDiaMes();
        $dia_inicial= new TkDateTime($dia_inicial);
        $dia_final= new TkDateTime($dia_final);
        $dias_reais=$dia_inicial->daysAfter($dia_final);
        return $dias_reais;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_evento Numero do evento.
     *
     * @return number
     *
     * @author alessandrobartels 12/08/2011
     */
    public function retornaContadorEvento($nr_evento){
        if ((int)$nr_evento === (int)$this->_nrEventoCorrente) {
            return $this->_ocorrenciasEventoCorrente;
        } else {
            $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
            $contador = $this->_itens_calculo->retornaContadorPorEvento($evento);
            return $contador;
        }
    }


    /**
     * Verifica a existencia de um evento no mes de competencia.
     *
     * @param integer $nr_evento Numero do evento.
     *
     * @return boolean
     *
     * @author alessandrobartels / bruno feliciano - 17/08/2011
     */
    public function verificaExistenciaEvento ($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $mes_competencia = $this->_calculo->getMesCompetencia();
        $indice = $this->_itens_calculo->procuraEvento($evento, $mes_competencia);

        $verifica = true;

        if ($indice === -1) {
            $verifica = false;
        }

        return $verifica;
    }


    /**
     * Enter description here ...
     *
     * @param float $valor Valor do item calculo (evento).
     *
     * @return void
     */
    public function atribuiValor($valor){
        $this->_valorEventoCorrente = $valor;
    }


    /**
     * Enter description here ...
     *
     * @param float $referencia Referencia do item calculo (evento).
     *
     * @return void
     */
    public function atribuiReferencia($referencia){
        $this->_referenciaEventoCorrente = $referencia;
    }


    /**
     * Enter description here ...
     *
     * @param integer $ocorrencias Numero de ocorrencias do item calculo (evento).
     *
     * @return void
     */
    public function atribuiOcorrencias($ocorrencias){
        $this->_ocorrenciasEventoCorrente = $ocorrencias;
    }


    /**
     * Totaliza os valores de uma lista de eventos.
     *
     * @param string  $listaNrEvento   Lista de numero de eventos.
     * @param integer $nrTipoMovimento Numero do tipo de movimento.
     * @param integer $deslocamento    Quantidade de meses de deslocamento da data.
     * @param string  $mesIncidencia   Mes de incidencia do evento.
     *
     * @return float;
     *
     * @author alessandrobartels / bruno feliciano - 16/08/2011
     */
    public function totalizaValorHistorico($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia = null){
        $valor = $this->_totalizaHistoricoPorTipo(
            $listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, FpaitecalcfolhaEntity::VALOR
        );
        return $valor;
    }


    /**
     * Totaliza as referencias de uma lista de eventos.
     *
     * @param string  $listaNrEvento   Lista de numero de eventos.
     * @param integer $nrTipoMovimento Numero do tipo de movimento.
     * @param integer $deslocamento    Quantidade de meses de deslocamento da data.
     * @param string  $mesIncidencia   Mes de incidencia do evento.
     *
     * @return float;
     *
     * @author alessandrobartels / bruno feliciano - 16/08/2011
     */
    public function totalizaReferenciaHistorico($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia = null){

        $referencia = $this->_totalizaHistoricoPorTipo(
            $listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, FpaitecalcfolhaEntity::REFERENCIA
        );
        return $referencia;
    }


    /*
    public function totalizaOcorrenciaHistorico($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia = null){

        $referencia = $this->_totalizaHistoricoPorTipo(
            $listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, FpaitecalcfolhaEntity::PARCELA
        );
        return $referencia;
    }
    */


    /**
     * Totaliza o valor de todos os eventos.
     *
     * @param string  $listaNrEvento   Lista de numero de eventos.
     * @param integer $nrTipoMovimento Numero do tipo de movimento.
     * @param integer $deslocamento    Quantidade de meses de deslocamento da data.
     * @param string  $mesIncidencia   Mes de incidencia do evento.
     * @param string  $tipo            Tipo de dados a ser totalizado (Ex.: Valor, Referencia ...).
     *
     * @return float
     *
     * @author alessandrobartels / bruno feliciano - 16/08/2011
     */
    private function _totalizaHistoricoPorTipo($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, $tipo){

        if ($mesIncidencia ===  null) {
            $mesIncidencia = $this->_calculo->getMesCompetencia();
        } else {
            $mesIncidencia = new TkDateTime($mesIncidencia);
            $mesIncidencia->setDay(1);
        }

        // Totaliza itens de um mesmo evento.
        $lista = $this->_retornaItensCalculosHistorico($listaNrEvento, $nrTipoMovimento, $mesIncidencia, $deslocamento);

        foreach ($lista as $item) {
            // @todo compatibilizar o tipo de referencia em DIA ou HORA. verificar.
            $valor += $item[$tipo];
        }

        return $valor;

    }


    /**
     * Totaliza as ocorrencias dos respectivos eventos e armazena em array.
     *
     * @param string  $listaNrEvento   Lista de numero de eventos.
     * @param integer $nrTipoMovimento Numero do tipo de movimento.
     * @param string  $mesIncidencia   Mes de incidencia do evento.
     * @param integer $deslocamento    Quantidade de meses de deslocamento da data.
     *
     * @return Ambigous <multitype:, void, NULL>
     *
     * @author alessandrobartels / bruno feliciano - 17/08/2011
     */
    private function _retornaItensCalculosHistorico($listaNrEvento, $nrTipoMovimento, $mesIncidencia, $deslocamento){

        /* Calcula o numero de meses entre o data de competencia e a data de incidencia.
         * O resultado sera usado para indexar a matriz de itens de calculo ja retornados da base.
         */
        $dtInicial = new TkDateTime($mesIncidencia);
        if ($deslocamento > 0) {
            $dtInicial->subMonth($deslocamento);
        }

        $dtFinal = new TkDateTime($mesIncidencia);

        $linha = $this->_calculo->getMesCompetencia()->monthsAfter($mesIncidencia);
        $coluna = $deslocamento;

        // Verifica se existe lista de itens calculos previamente calculada.
        if (count($this->_calculo->listaCalculada[$linha][$coluna]) > 0) {
            $listaNrCalcFolha = $this->_calculo->listaCalculada[$linha][$coluna];
        } else {
            $calculoFolha = new FpacalculofolhaEntity();

            $listaNrCalcFolha = $calculoFolha->retornaCalculoFolhaPeriodo($this->_calculo->getNrorg(),
                                $nrTipoMovimento, $dtInicial, $dtFinal);

            // Grava lista para verificacoes posteriores.
            $this->_calculo->listaCalculada[$linha][$coluna] = $listaNrCalcFolha;
        }

        $itemCalculo = new FpaitecalcfolhaEntity();
        $listaItensCalculos = $itemCalculo->totalizaItensHistorico(
            $this->_calculo->getVinculo(), $listaNrCalcFolha, $listaNrEvento
        );
        return $listaItensCalculos;

    }


    /**
     * Enter description here ...
     *
     * @return float
     */
    public function totalizaProvento(){
        $total = $this->_retornaValorTotalProventoDesconto();
        return $total[FpaeventomEntity::IDTIPOEVENTO_PROVENTO];
    }


    /**
     * Enter description here ...
     *
     * @return float
     */
    public function totalizaDesconto(){
        $total = $this->_retornaValorTotalProventoDesconto();
        return $total[FpaeventomEntity::IDTIPOEVENTO_DESCONTO];
    }


    /**
     * Enter description here ...
     *
     * @return float
     */
    private function _retornaValorTotalProventoDesconto(){
        $total = $this->_itens_calculo->retornaTotais($this->_calculo->getEventom());
        return $total;
    }


    /* ##############################   VINCULO   ##############################*/


    /**
     * Retorna o conteudo de campo especificado do vinculo.
     *
     * @param string $descricao Nome do campo da tabela referente ao vinculo (mestre ou historico).
     *
     * @return mixed
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     */
    public function retornaDadosVinculo($descricao){
        $objeto = $this->_calculo->getVinculo();
        return $this->_retornaDados($descricao, $objeto);

    }


    /**
     * Usado para retornar dados de qualquer entidade (TkEntity), inclusive mestre/historico.
     *
     * @param string   $descricao Nome do campo.
     * @param TkEntity $objeto    Instancia de herdada de TkEntity.
     *
     * @return mixed
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     */
    private function _retornaDados($descricao, TkEntity $objeto){

        $nomeColuna = strtolower($descricao);
        $nomeColuna[0] = strtoupper($nomeColuna[0]);
        $metodo = 'get'.$nomeColuna.'();';

        $resultado = '';
        /* Pesquisa a existencia da coluna na entidade mestre */
        $objetoMestre = $objeto->getDaoAttribute(strtolower($nomeColuna));
        if (isset($objetoMestre) === true) {
            $resultado = 'return $objeto->'.$metodo;
        } else {
            /* Pesquisa a existencia da coluna na entidade historica */
            $objetoHistorico = $objeto->historico->getDaoAttribute(strtolower($nomeColuna));
            if (isset($objetoHistorico) === true) {
                $resultado = 'return $objeto->historico->'.$metodo;
            } else {
                tk_warning('Coluna '.$descricao.' inexistente na entidade.');
            }
        }

        return eval($resultado);

    }



    /* ##############################   EXECUCAO DE REGRAS   ############################## */


    /**
     * Executa regra e retorna resultado.
     *
     * @param string  $codigo Codigo de execucao da regra.
     * @param boolean $debug  Flag para exibir o debug.
     *
     * @return mixed
     *
     * @see TeknisaAbstractRegra::executaRegra()
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     */
    public function executaRegra($codigo, $debug = false){
        $valor = parent::executaRegra($codigo, $debug);
        return $valor;
    }


    /**
     * Executa a regra.
     *
     * @param boolean         $debug Flag para exibir o debug.
     * @param FparegramEntity $regra Instancia de FparegramEntity agregado com FparegrahEntity.
     *
     * @return void
     *
     * @author alessandrobartels - 05/12/2011
     */
    private function _preparaRegra($debug, FparegramEntity $regra){

        if ($debug === true) {
            // @todo criar as constantes na fparegram.
            if ($regra->getIddebug() === 'N') {
                $debug = false;
            }
        }

        $this->_eventoCorrente = $this->_calculo->getEventom()->retornaEvento($regra->historico->getNreventom());

        if (is_object($this->_eventoCorrente) === false) {
            /* tk_warning('Evento nao encontrado.'); */
            return;
        }

        $this->_atribuiVariavelEventoCorrente($this->_eventoCorrente->getNreventom());

        $this->_nrEventoCorrente = $this->_eventoCorrente->getNreventom();

        // Pode existir mais de uma regra para o evento. Metodo abaixo, retorna as posicoes dos eventos na lista.
        $indices = $this->_itens_calculo->retornaItens($this->_eventoCorrente);
        // Verifica se o evento é variavel ou de sistema,
        if (count($indices) > 0) {
            foreach ($indices as $indice) {
                $this->_mesOcorrenciaEventoCorrente = $this->_itens_calculo
                    ->retornaDtocorrencia($indice);
                $this->_referenciaEventoCorrente = $this->_itens_calculo
                    ->retornaReferencia($indice);
                $this->_valorEventoCorrente = $this->_itens_calculo
                    ->getVreventofolha($indice);
                $this->_ocorrenciasEventoCorrente= $this->_itens_calculo
                    ->retornaContador($indice);
                $valor = $this->executaRegra($regra->historico->getDsregra(), $debug);
                if ($valor === ABORT) {
                    continue;
                } else {
                    $this->_valorEventoCorrente = (float)$valor;
                    $this->_itens_calculo->geraEvento(
                        $this->_eventoCorrente, $this->_mesOcorrenciaEventoCorrente, $this->_valorEventoCorrente,
                        $this->_referenciaEventoCorrente, $this->_ocorrenciasEventoCorrente
                    );
                }
            }
        } else {
            $tpOcorrencia = $this->_eventoCorrente->historico->getIdtipoocorreven();
            if ($tpOcorrencia === FpaeventohEntity::IDTIPOOCORREVEN_SISTEMA) {
                $this->_mesOcorrenciaEventoCorrente = $this->_calculo->getMesCompetencia();
                $this->_referenciaEventoCorrente = 0;
                $this->_valorEventoCorrente = 0;
                $this->_ocorrenciasEventoCorrente = 1;
                $valor = $this->executaRegra($regra->historico->getDsregra(), $debug);
                /* @todo tratar */
                $this->_valorEventoCorrente = (float)$valor;

                $this->_itens_calculo->geraEvento(
                    $this->_eventoCorrente, $this->_mesOcorrenciaEventoCorrente, $this->_valorEventoCorrente,
                    $this->_referenciaEventoCorrente, $this->_ocorrenciasEventoCorrente
                );
            }
        }
    }


    /**
     * Executa regra por ponto de regra.
     *
     * @param boolean $debug         Flag para exibir o debug.
     * @param integer $nrPtRegra     Numero do ponto de regra.
     * @param integer $nrTipoCalculo Numero do tipo de calculo.
     *
     * @return void
     *
     * @author alessandrobartels / bruno feliciano - 12/08/2011
     *         alessandrobartels - 06/12/2011 - Faz loop nos relacionamentos por tipo de calculo.
     */
    public function executaPontoRegra($debug = false, $nrPtRegra, $nrTipoCalculo,FpaitecalcfolhaEntity $itens_calculo){

        $this->_itens_calculo=$itens_calculo; // BORBA 01/07/2013 - Antes a regra executava de forma fixa no itemcalculoprincipal do calculo.
        //A rescisão precisa executar regra em ferias 13 e aviso previo.
        // Procura o objeto na lista na posicao do ponteiro. Se lista vazia, inicia lista.
        $objTpCalcRegras = $this->_calculo->getTpCalcRegra()->procuraTpCalcRegra($nrTipoCalculo);
        foreach($objTpCalcRegras as $objTpCalcRegra) {
            if ((int)$nrPtRegra === (int)$objTpCalcRegra->getNrptregra()) {
                $regra = $this->_calculo->getRegram()->retornaRegra($objTpCalcRegra->getNrregram());
                if (is_object($regra) === true) {
                    $this->_preparaRegra($debug, $regra);
                }
            }
        }

    }

    public function getDiasEntre($dataInicial, $dataFinal) {
        $dataInicio = new TkDateTime($dataInicial);
        return $dataInicio->daysUntil($dataFinal);
    }

    public function getDataCompetencia() {
        return \FPA::getDtCompetencia();
    }

    public function getUltimoDiaMes() {	
		return \FPA::getUltimoDiaMes();
    }


    public function getTomador($nrestrutgeren)  {
        $nrorg = FPA::getNrOrgTrabalho();

        return Gpevinculom::getTomador($nrorg, $nrestrutgeren);
    }

}